from typing import List, Union

from docarray import BaseDoc, DocList
from docarray.base_doc.doc import BaseDocWithoutId
from docarray.typing import NdArray
from pydantic import Field, BaseModel

from api_schemas.base import BaseInputModel


class ExecutorUsage(BaseDoc):
    """The usage of the embedding services to report, e.g. number of tokens in case of text input"""
    total_tokens: int = Field(description='The number of tokens used to embed the input text')


# EXECUTOR MODELS
## Model to be imported by the Executor and used by the Universal API to communicate with it
class TextDoc(BaseDoc):
    """Document containing a text field"""
    text: str


## Model to be imported by the Executor and used by the Universal API to communicate with it
class EmbeddingDoc(BaseDoc):
    """Document to be returned by the embedding backend, containing the embedding vector and the token usage for the
    corresponding input texts"""
    embedding: NdArray = Field(description='The embedding of the texts', default=[])
    usage: ExecutorUsage


# UNIVERSAL API MODELS (mimic OpenAI API)
class TextEmbeddingInput(BaseInputModel):
    """The input to the API for text embedding. OpenAI compatible"""
    input: Union[List[str], str] = Field(
        description='List of texts to embed. The length of the list must be between `1` and `100`',
        min_items=1,
        max_items=100,
    )

    class Config(BaseDoc.Config):
        allow_population_by_field_name = True
        arbitrary_types_allowed = True
        schema_extra = {
            "example": {
                "model": "jina-embedding-s-en-v1",
                "inputs": ["Hello, world!"],
            },
        }


class EmbeddingObject(BaseDocWithoutId):
    """Embedding object. OpenAI compatible"""
    object: str = 'embedding'
    index: int = Field(description='The index of the embedding output, corresponding to the index in the list of inputs')
    embedding: NdArray = Field(description='The embedding of the texts', default=[])

    class Config(BaseDocWithoutId.Config):
        allow_population_by_field_name = True
        arbitrary_types_allowed = True
        schema_extra = {
            "example": {
                "object": "embedding",
                "index": 0,
                "embedding": [0.1, 0.2, 0.3],
            }
        }


class Usage(BaseModel):
    total_tokens: int = Field(description='The number of tokens used by all the texts in the input')
    prompt_tokens: int = Field(description='Same as total_tokens')


class ModelEmbeddingOutput(BaseInputModel):
    """Output of the embedding service"""
    object: str = 'list'
    data: DocList[EmbeddingObject] = Field(description='A list of Embedding Objects returned by the embedding service')
    usage: Usage = Field(description='Total usage of the request. Sums up the usage from each individual input')
